#define PROGNAME "astreamer"
#define ALSA_PCM_NEW_HW_PARAMS_API
#define BASE_ADDRESS 3758161665L /* 224.0.255.1 */
#define BASE_PORT 6000
#include <alsa/asoundlib.h>
#include <plib/socket.h>

static BOOL exit_flag = FALSE;

static BOOL is_noise(const void * buffer, size_t amount, int silence_level) {

	REGISTER short * start = (short *)buffer;
	short * end   = start + (amount>>1);
	REGISTER short value;
	REGISTER long sum;

	sum=0;
	while (start != end) {
//	    value = ntohs(*start);
	    value = *start;
	    if(value < 0)
		sum -= value;
	    else
		sum += value;
	    start++;
	}
	return ((int)(sum/amount) > silence_level);
}

void term_signal(int signum) {
    exit_flag = TRUE;
}

void int_signal(int signum) {
    exit_flag = TRUE;
}


static int msock_open( void) {
    int	fd;
    int 	i;
    struct in_addr iface;

    if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
	log_err("sock_open: socket");
	return (-1);
    }
    iface.s_addr = htonl(INADDR_ANY);
    if ( setsockopt( fd, IPPROTO_IP, IP_MULTICAST_IF, (char *)&iface, sizeof(iface)) < 0) {
	log_err("sock_open: could not setup multicast interface");
	close( fd);
	return (-1);
    }
    unsigned char ttl = (unsigned char)255;
    setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
    i = 1024;
    log_debug("Setting socket recieve buf to %d", i);
    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &i, sizeof(i)) < 0)
	log_debug("setsockopt(SO_RCVBUF)");
    return fd;
}

static void usage( void) {
    fprintf(stderr, "Usage: astreamer NUMBER [SILENCE_LEVEL]\n");
    exit(1);
}

int main( int argc, char * argv[]) {
    int rc, fd;
    snd_pcm_t *handle;
    snd_pcm_hw_params_t *params;
    unsigned int val;
    int dir = 0;
    snd_pcm_uframes_t frames;
    char buffer[1360];
    struct sockaddr_in maddr;
    size_t size;
    int silence = 60;


    plib_init();
    log_open(PROGNAME, L_INFO|L_NOTICE|L_DBG|L_ERR);
    if ( argc < 2)
	usage();
    if ( argc > 2)
	silence = atoi(argv[2]);
    if ((fd = msock_open()) < 0) {
	log_err("sock_open: socket");
    }
    memset((void *)&maddr, 0, sizeof(maddr));
    maddr.sin_addr.s_addr = htonl(BASE_ADDRESS+atoi(argv[1]));
    maddr.sin_port = htons(BASE_PORT+((unsigned short)atoi(argv[1])*10));
    maddr.sin_family = AF_INET;
    /* Open PCM device for recording (capture). */
    rc = snd_pcm_open(&handle, "default", SND_PCM_STREAM_CAPTURE, 0);
    if (rc < 0)
	log_err("unable to open pcm device: %s\n", snd_strerror(rc));
    /* Allocate a hardware parameters object. */
    snd_pcm_hw_params_alloca(&params);
    /* Fill it in with default values. */
    snd_pcm_hw_params_any(handle, params);
    /* Set the desired hardware parameters. */
    /* Interleaved mode */
    snd_pcm_hw_params_set_access(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED);

    /* Use a-Law format */
    snd_pcm_hw_params_set_format(handle, params, SND_PCM_FORMAT_S16_LE);
    /* One channel (mono) */
    snd_pcm_hw_params_set_channels(handle, params, 1);
    /* 8000 bits/second sampling rate (phone quality) */
    val = 8000;
    snd_pcm_hw_params_set_rate_near(handle, params, &val, &dir);
    /* Set period size to 440 frames. */
    frames = 440;
    snd_pcm_hw_params_set_period_size_near(handle, params, &frames, &dir);
    /* Write the parameters to the driver */
    rc = snd_pcm_hw_params(handle, params);
    if (rc < 0)
	log_err( "unable to set hw parameters: %s\n", snd_strerror(rc));

    /* Use a buffer large enough to hold one period */
    snd_pcm_hw_params_get_period_size(params, &frames, &dir);
    size = frames * 2;
    /* We want to loop for 5 seconds */
    snd_pcm_hw_params_get_period_time(params, &val, &dir);

    port_signal(SIGINT, int_signal);
    port_signal(SIGTERM, term_signal);
    snd_pcm_start(handle);

    while (!exit_flag) {
	rc = snd_pcm_readi(handle, buffer, frames);
	if (rc == -EPIPE) {
	    /* EPIPE means overrun */
	    log_debug("overrun occurred\n");
	    snd_pcm_recover(handle, rc, 0);
	} else if (rc == -EBADFD) {
	    log_debug("Wrong state\n");
	    snd_pcm_recover(handle, rc, 0);
	} else if (rc == -ESTRPIPE) {
	    log_debug("Suspended\n");
	    snd_pcm_recover(handle, rc, 0);
	} else if (rc < 0) {
	    log_debug("error from read: %s\n", snd_strerror(rc));
	} else if (rc != (int)frames) {
	    log_debug("short read, read %d frames\n", rc);
	}
	if  (is_noise( buffer, size, silence)) {
	    rc = sendto(fd, buffer, size, 0, (void *)&maddr, sizeof(maddr));
	    if (rc != size)
		log_debug("short write: wrote %d bytes. Errno is %d. %s\n", rc, errno, strerror(errno));
	}
    }
    snd_pcm_drain(handle);
    snd_pcm_close(handle);
    close(fd);
    plib_close();
    return 0;
}
